@@ -33,10 +33,10 @@ int main()
    /* Add stuff to the distinguished name
     */
    gnutls_x509_crq_set_dn_by_oid(crq, GNUTLS_OID_X520_COUNTRY_NAME,
-				     "GR", 2);
+				     0, "GR", 2);
 
    gnutls_x509_crq_set_dn_by_oid(crq, GNUTLS_OID_X520_COMMON_NAME,
-				     "Nikos", strlen("Nikos"));
+				     0, "Nikos", strlen("Nikos"));
 
    /* Set the request version.
     */
