@@ -37,13 +37,25 @@
 extern "C" {
 #endif
 
-#define LIBGNUTLS_VERSION "0.9.97"
+#define LIBGNUTLS_VERSION "0.9.98"
 
 /* For backwards compatibility */
 #include <gnutls/compat4.h>
 
+#ifndef HAVE_SIZE_T
+# define HAVE_SIZE_T
 #include <sys/types.h>
+#endif
+
+#ifndef HAVE_SSIZE_T
+# define HAVE_SSIZE_T
+
+#endif
+
+#ifndef HAVE_TIME_T
+# define HAVE_TIME_T
 #include <time.h>
+#endif
 
 #define GNUTLS_CIPHER_RIJNDAEL_128_CBC GNUTLS_CIPHER_AES_128_CBC
 #define GNUTLS_CIPHER_RIJNDAEL_256_CBC GNUTLS_CIPHER_AES_256_CBC
@@ -63,11 +75,15 @@ typedef enum gnutls_kx_algorithm { GNUTLS_KX_RSA=1, GNUTLS_KX_DHE_DSS,
 
 typedef enum gnutls_credentials_type { GNUTLS_CRD_CERTIFICATE=1, GNUTLS_CRD_ANON, GNUTLS_CRD_SRP } gnutls_credentials_type;
 
-typedef enum gnutls_mac_algorithm { GNUTLS_MAC_NULL=1, GNUTLS_MAC_MD5, GNUTLS_MAC_SHA } gnutls_mac_algorithm;
+typedef enum gnutls_mac_algorithm { GNUTLS_MAC_NULL=1, 
+	GNUTLS_MAC_MD5, GNUTLS_MAC_SHA, GNUTLS_MAC_RMD160
+} gnutls_mac_algorithm;
 
 /* The enumerations here should have the same value with gnutls_mac_algorithm.
  */
-typedef enum gnutls_digest_algorithm { GNUTLS_DIG_NULL=1, GNUTLS_DIG_MD5, GNUTLS_DIG_SHA, GNUTLS_DIG_MD2 } gnutls_digest_algorithm;
+typedef enum gnutls_digest_algorithm { GNUTLS_DIG_NULL=1, GNUTLS_DIG_MD5, 
+	GNUTLS_DIG_SHA, GNUTLS_DIG_RIPEMD160 
+} gnutls_digest_algorithm;
 
 /* exported for other gnutls headers. This is the maximum number
  * of algorithms (ciphers, kx or macs). 
