@@ -21,16 +21,6 @@
 #ifndef DEFINES_H
 # define DEFINES_H
 
-#ifndef __GNUC__
-# if HAVE_ALLOCA_H
-#  include <alloca.h>
-# else
-#  ifdef _AIX
-#pragma alloca
-#  endif
-# endif
-#endif
-
 #include <config.h>
 
 #ifdef STDC_HEADERS
@@ -41,11 +31,17 @@
 #endif
 
 #ifdef NO_TIME_T
+# define HAVE_TIME_T
 typedef unsigned int time_t;
 #endif
 
 #ifdef NO_SIZE_T
+# define HAVE_SIZE_T
 typedef unsigned int size_t;
+#endif
+
+#ifdef NO_SSIZE_T
+# define HAVE_SSIZE_T
 typedef int ssize_t;
 #endif
 
