@@ -163,6 +163,7 @@ typedef struct gnutls_hash_entry gnutls_hash_entry;
 static const gnutls_hash_entry hash_algorithms[] = {
 	{"SHA", GNUTLS_MAC_SHA, 20},
 	{"MD5", GNUTLS_MAC_MD5, 16},
+	{"RIPEMD160", GNUTLS_MAC_RMD160, 20},
 	{"NULL", GNUTLS_MAC_NULL, 0},
 	{0, 0, 0}
 };
@@ -313,20 +314,35 @@ typedef struct {
 
 #define GNUTLS_DHE_DSS_3DES_EDE_CBC_SHA { 0x00, 0x13 }
 
+
 /* gnutls private extensions: 
  */
 #define GNUTLS_DHE_DSS_TWOFISH_128_CBC_SHA { 0xFF, 0x54 }
 
+
+/* draft-ietf-tls-openpgp-keys-04:
+ */
+#define GNUTLS_DHE_DSS_3DES_EDE_CBC_RMD { 0x00, 0x72 }
+#define GNUTLS_DHE_RSA_3DES_EDE_CBC_RMD { 0x00, 0x77 }
+#define GNUTLS_DHE_DSS_AES_256_CBC_RMD { 0x00, 0x73 }
+#define GNUTLS_DHE_DSS_AES_128_CBC_RMD { 0x00, 0x74 }
+#define GNUTLS_DHE_RSA_AES_128_CBC_RMD { 0x00, 0x78 }
+#define GNUTLS_DHE_RSA_AES_256_CBC_RMD { 0x00, 0x79 }
+#define GNUTLS_RSA_3DES_EDE_CBC_RMD { 0x00, 0x7C }
+#define GNUTLS_RSA_AES_128_CBC_RMD { 0x00, 0x7D }
+#define GNUTLS_RSA_AES_256_CBC_RMD { 0x00, 0x7E }
+
+
 /* draft-ietf-tls-56-bit-ciphersuites-01:
  */
 #define GNUTLS_DHE_DSS_ARCFOUR_SHA { 0x00, 0x66 }
 
+
 /* rfc3268: 
  */
 #define GNUTLS_DHE_DSS_AES_256_CBC_SHA { 0x00, 0x38 }
 #define GNUTLS_DHE_DSS_AES_128_CBC_SHA { 0x00, 0x32 }
 
-
 /** DHE RSA 
  **/
 #define GNUTLS_DHE_RSA_3DES_EDE_CBC_SHA { 0x00, 0x16 }
@@ -411,7 +427,15 @@ static const gnutls_cipher_suite_entry cs_algorithms[] = {
 	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DHE_DSS_AES_256_CBC_SHA,
 				  GNUTLS_CIPHER_AES_256_CBC, GNUTLS_KX_DHE_DSS,
 				  GNUTLS_MAC_SHA, GNUTLS_SSL3),
-
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DHE_DSS_3DES_EDE_CBC_RMD,
+				  GNUTLS_CIPHER_3DES_CBC, GNUTLS_KX_DHE_DSS,
+				  GNUTLS_MAC_RMD160, GNUTLS_TLS1),
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DHE_DSS_AES_128_CBC_RMD,
+				  GNUTLS_CIPHER_AES_128_CBC, GNUTLS_KX_DHE_DSS,
+				  GNUTLS_MAC_RMD160, GNUTLS_TLS1),
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DHE_DSS_AES_256_CBC_RMD,
+				  GNUTLS_CIPHER_AES_256_CBC, GNUTLS_KX_DHE_DSS,
+				  GNUTLS_MAC_RMD160, GNUTLS_TLS1),
 	/* DHE_RSA */
 	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DHE_RSA_TWOFISH_128_CBC_SHA,
 				  GNUTLS_CIPHER_TWOFISH_128_CBC, GNUTLS_KX_DHE_RSA,
@@ -425,7 +449,15 @@ static const gnutls_cipher_suite_entry cs_algorithms[] = {
 	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DHE_RSA_AES_256_CBC_SHA,
 				  GNUTLS_CIPHER_AES_256_CBC, GNUTLS_KX_DHE_RSA,
 				  GNUTLS_MAC_SHA, GNUTLS_SSL3),
-
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DHE_RSA_3DES_EDE_CBC_RMD,
+				  GNUTLS_CIPHER_3DES_CBC, GNUTLS_KX_DHE_RSA,
+				  GNUTLS_MAC_RMD160, GNUTLS_TLS1),
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DHE_RSA_AES_128_CBC_RMD,
+				  GNUTLS_CIPHER_AES_128_CBC, GNUTLS_KX_DHE_RSA,
+				  GNUTLS_MAC_RMD160, GNUTLS_TLS1),
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DHE_RSA_AES_256_CBC_RMD,
+				  GNUTLS_CIPHER_AES_256_CBC, GNUTLS_KX_DHE_RSA,
+				  GNUTLS_MAC_RMD160, GNUTLS_TLS1),
 	/* RSA */
 	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_RSA_NULL_MD5,
 				  GNUTLS_CIPHER_NULL,
@@ -453,7 +485,15 @@ static const gnutls_cipher_suite_entry cs_algorithms[] = {
 	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_RSA_TWOFISH_128_CBC_SHA,
 				  GNUTLS_CIPHER_TWOFISH_128_CBC, GNUTLS_KX_RSA,
 				  GNUTLS_MAC_SHA, GNUTLS_TLS1),
-
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_RSA_3DES_EDE_CBC_RMD,
+				  GNUTLS_CIPHER_3DES_CBC,
+				  GNUTLS_KX_RSA, GNUTLS_MAC_RMD160, GNUTLS_TLS1),
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_RSA_AES_128_CBC_RMD,
+				  GNUTLS_CIPHER_AES_128_CBC, GNUTLS_KX_RSA,
+				  GNUTLS_MAC_RMD160, GNUTLS_TLS1),
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_RSA_AES_256_CBC_RMD,
+				  GNUTLS_CIPHER_AES_256_CBC, GNUTLS_KX_RSA,
+				  GNUTLS_MAC_RMD160, GNUTLS_TLS1),
 	{0, {{0,0}}, 0, 0, 0, 0}
 };
 
