@@ -34,10 +34,10 @@ gcry_error_t err = GPG_ERR_GENERAL; /* doesn't matter */
 
 
 	switch (cipher) {
-	case GNUTLS_CIPHER_RIJNDAEL_128_CBC:
+	case GNUTLS_CIPHER_AES_128_CBC:
 		err = gcry_cipher_open(&ret, GCRY_CIPHER_RIJNDAEL, GCRY_CIPHER_MODE_CBC, 0);
 		break;
-	case GNUTLS_CIPHER_RIJNDAEL_256_CBC:
+	case GNUTLS_CIPHER_AES_256_CBC:
 		err = gcry_cipher_open(&ret, GCRY_CIPHER_RIJNDAEL256, GCRY_CIPHER_MODE_CBC, 0);
 		break;
 	case GNUTLS_CIPHER_TWOFISH_128_CBC:
