@@ -30,38 +30,38 @@
 
 GNUTLS_HASH_HANDLE _gnutls_hash_init(gnutls_mac_algorithm algorithm)
 {
-	GNUTLS_MAC_HANDLE ret;
+	GNUTLS_MAC_HANDLE ret = NULL;
 	gcry_error_t result;
 
+	ret = gnutls_malloc(sizeof(GNUTLS_MAC_HANDLE_INT));
+	if (ret == NULL) {
+		gnutls_assert();
+		return GNUTLS_HASH_FAILED;
+	}
+
+	ret->algorithm = algorithm;
+
 	switch (algorithm) {
 	case GNUTLS_MAC_SHA:
-		ret = gnutls_malloc(sizeof(GNUTLS_MAC_HANDLE_INT));
-		if (ret == NULL)
-			return GNUTLS_HASH_FAILED;
 		result = gcry_md_open( &ret->handle, GCRY_MD_SHA1, 0);
-		if (result) {
-			gnutls_free(ret);
-			ret = GNUTLS_HASH_FAILED;
-		}
 		break;
-
 	case GNUTLS_MAC_MD5:
-		ret = gnutls_malloc(sizeof(GNUTLS_MAC_HANDLE_INT));
-		if (ret == NULL)
-			return GNUTLS_HASH_FAILED;
 		result = gcry_md_open( &ret->handle, GCRY_MD_MD5, 0);
-		if (result) {
-			gnutls_free(ret);
-			ret = GNUTLS_HASH_FAILED;
-		}
 		break;
-
+	case GNUTLS_MAC_RMD160:
+		result = gcry_md_open( &ret->handle, GCRY_MD_RMD160, 0);
+		break;
 	default:
+		gnutls_assert();
+		gnutls_free( ret);
 		ret = GNUTLS_HASH_FAILED; break;
 	}
 
-	if (ret != GNUTLS_HASH_FAILED)
-		ret->algorithm = algorithm;
+	if (result) {
+		gnutls_assert();
+		gnutls_free(ret);
+		ret = GNUTLS_HASH_FAILED;
+	}
 
 	return ret;
 }
@@ -77,6 +77,9 @@ int _gnutls_hash_get_algo_len(gnutls_mac_algorithm algorithm)
 	case GNUTLS_MAC_MD5:
 		ret = gcry_md_get_algo_dlen(GCRY_MD_MD5);
 		break;
+	case GNUTLS_MAC_RMD160:
+		ret = gcry_md_get_algo_dlen(GCRY_MD_MD5);
+		break;
 	default:
 		ret = 0; break;
 	}
@@ -118,7 +121,7 @@ GNUTLS_HASH_HANDLE _gnutls_hash_copy(GNUTLS_HASH_HANDLE handle)
 
 void _gnutls_hash_deinit(GNUTLS_HASH_HANDLE handle, void *digest)
 {
-	char *mac;
+	opaque *mac;
 	int maclen;
 
 	maclen = gcry_md_get_algo_dlen(gcry_md_get_algo(handle->handle));
@@ -140,31 +143,28 @@ GNUTLS_MAC_HANDLE _gnutls_hmac_init(gnutls_mac_algorithm algorithm,
 	GNUTLS_MAC_HANDLE ret;
 	gcry_error_t result;
 
+	ret = gnutls_malloc(sizeof(GNUTLS_MAC_HANDLE_INT));
+	if (ret == NULL)
+		return GNUTLS_MAC_FAILED;
+
 	switch (algorithm) {
 	case GNUTLS_MAC_SHA:
-		ret = gnutls_malloc(sizeof(GNUTLS_MAC_HANDLE_INT));
-		if (ret == NULL)
-			return GNUTLS_MAC_FAILED;
-
 		result = gcry_md_open(&ret->handle, GCRY_MD_SHA1, GCRY_MD_FLAG_HMAC);
-
-		if (result)
-			ret = GNUTLS_MAC_FAILED;
 		break;
 	case GNUTLS_MAC_MD5:
-		ret = gnutls_malloc(sizeof(GNUTLS_MAC_HANDLE_INT));
-		if (ret == NULL)
-			return GNUTLS_MAC_FAILED;
-
 		result = gcry_md_open(&ret->handle, GCRY_MD_MD5, GCRY_MD_FLAG_HMAC);
-
-		if (result)
-			ret = GNUTLS_MAC_FAILED;
+		break;
+	case GNUTLS_MAC_RMD160:
+		result = gcry_md_open(&ret->handle, GCRY_MD_RMD160, GCRY_MD_FLAG_HMAC);
 		break;
 	default:
+		gnutls_free(ret);
 		ret = GNUTLS_MAC_FAILED;
 	}
 
+	if (result)
+		ret = GNUTLS_MAC_FAILED;
+
 	if (ret != GNUTLS_MAC_FAILED) {
 		gcry_md_setkey(ret->handle, key, keylen);
 
@@ -173,8 +173,6 @@ GNUTLS_MAC_HANDLE _gnutls_hmac_init(gnutls_mac_algorithm algorithm,
 		ret->keysize = keylen;
 	}
 
-
-
 	return ret;
 }
 
@@ -194,7 +192,7 @@ int _gnutls_hmac(GNUTLS_MAC_HANDLE handle, const void *text, size_t textlen)
 
 void _gnutls_hmac_deinit(GNUTLS_MAC_HANDLE handle, void *digest)
 {
-	char *mac;
+	opaque *mac;
 	int maclen;
 
 	maclen = gcry_md_get_algo_dlen(gcry_md_get_algo(handle->handle));
@@ -210,26 +208,33 @@ void _gnutls_hmac_deinit(GNUTLS_MAC_HANDLE handle, void *digest)
 	gnutls_free(handle);
 }
 
+inline static int get_padsize( gnutls_mac_algorithm algorithm)
+{
+	switch (algorithm) {
+	case GNUTLS_MAC_MD5:
+		return 48;
+	case GNUTLS_MAC_SHA:
+		return 40;
+	default:
+		return 0;
+	}
+}
+
 GNUTLS_MAC_HANDLE _gnutls_mac_init_ssl3(gnutls_mac_algorithm algorithm, void *key,
 					int keylen)
 {
 	GNUTLS_MAC_HANDLE ret;
-	char ipad[48];
+	opaque ipad[48];
 	int padsize;
 
-	switch (algorithm) {
-	case GNUTLS_MAC_MD5:
-		padsize = 48;
-		break;
-	case GNUTLS_MAC_SHA:
-		padsize = 40;
-		break;
-	default:
-		padsize = 0;
-	}
-	if (padsize > 0) {
-		memset(ipad, 0x36, padsize);
+	padsize = get_padsize( algorithm);
+	if (padsize == 0) {
+		gnutls_assert();
+		return GNUTLS_MAC_FAILED;
 	}
+
+	memset(ipad, 0x36, padsize);
+
 	ret = _gnutls_hash_init(algorithm);
 	if (ret != GNUTLS_HASH_FAILED) {
 		ret->key = key;
@@ -247,24 +252,18 @@ void _gnutls_mac_deinit_ssl3(GNUTLS_MAC_HANDLE handle, void *digest)
 {
 	opaque ret[MAX_HASH_SIZE];
 	GNUTLS_MAC_HANDLE td;
-	char opad[48];
+	opaque opad[48];
 	int padsize;
 	int block;
 
-	switch (handle->algorithm) {
-	case GNUTLS_MAC_MD5:
-		padsize = 48;
-		break;
-	case GNUTLS_MAC_SHA:
-		padsize = 40;
-		break;
-	default:
-		padsize = 0;
-	}
-	if (padsize > 0) {
-		memset(opad, 0x5C, padsize);
+	padsize = get_padsize( handle->algorithm);
+	if (padsize == 0) {
+		gnutls_assert();
+		return;
 	}
 
+	memset(opad, 0x5C, padsize);
+
 	td = _gnutls_hash_init(handle->algorithm);
 	if (td != GNUTLS_MAC_FAILED) {
 		if (handle->keysize > 0)
@@ -285,26 +284,20 @@ void _gnutls_mac_deinit_ssl3_handshake(GNUTLS_MAC_HANDLE handle,
 {
 	opaque ret[MAX_HASH_SIZE];
 	GNUTLS_MAC_HANDLE td;
-	char opad[48];
-	char ipad[48];
+	opaque opad[48];
+	opaque ipad[48];
 	int padsize;
 	int block;
 
-	switch (handle->algorithm) {
-	case GNUTLS_MAC_MD5:
-		padsize = 48;
-		break;
-	case GNUTLS_MAC_SHA:
-		padsize = 40;
-		break;
-	default:
-		padsize = 0;
-	}
-	if (padsize > 0) {
-		memset(opad, 0x5C, padsize);
-		memset(ipad, 0x36, padsize);
+	padsize = get_padsize( handle->algorithm);
+	if (padsize == 0) {
+		gnutls_assert();
+		return;
 	}
 
+	memset(opad, 0x5C, padsize);
+	memset(ipad, 0x36, padsize);
+
 	td = _gnutls_hash_init(handle->algorithm);
 	if (td != GNUTLS_HASH_FAILED) {
 		if (key_size > 0)
@@ -324,11 +317,11 @@ void _gnutls_mac_deinit_ssl3_handshake(GNUTLS_MAC_HANDLE handle,
 	}
 }
 
-static int ssl3_sha(int i, char *secret, int secret_len, char *random,
+static int ssl3_sha(int i, opaque *secret, int secret_len, opaque *random,
 		    int random_len, void *digest)
 {
 	int j;
-	char text1[26];
+	opaque text1[26];
 
 	GNUTLS_HASH_HANDLE td;
 
@@ -350,7 +343,7 @@ static int ssl3_sha(int i, char *secret, int secret_len, char *random,
 	return 0;
 }
 
-static int ssl3_md5(int i, char *secret, int secret_len, char *random,
+static int ssl3_md5(int i, opaque *secret, int secret_len, opaque *random,
 		    int random_len, void *digest)
 {
 	opaque tmp[MAX_HASH_SIZE];
@@ -413,7 +406,7 @@ int _gnutls_ssl3_generate_random(void *secret, int secret_len,
 				 int ret_bytes, opaque * ret)
 {
 	int i = 0, copy, output_bytes;
-	char digest[MAX_HASH_SIZE];
+	opaque digest[MAX_HASH_SIZE];
 	int block = _gnutls_hash_get_algo_len(GNUTLS_MAC_MD5);
 	int result, times;
 
