@@ -185,10 +185,11 @@ int _gnutls_string_append_data( gnutls_string* dest, const void * data, size_t d
  * If the buffer does not have enough space to hold the string retuns
  * NULL.
  */
-char *_gnutls_bin2hex(const opaque *old, size_t oldlen,
+char *_gnutls_bin2hex(const void *_old, size_t oldlen,
 	char *buffer, size_t buffer_size)
 {
 	unsigned int i, j;
+	opaque *old = _old;
 
 	if ((oldlen * 2) + 1 > buffer_size) return NULL;
 
