@@ -77,6 +77,33 @@ int i = 0;
 	return 0;
 }
 
+/**
+  * gnutls_x509_dn_oid_known - This function will return true if the given OID is known
+  * @oid: holds an Object Identifier in a null terminated string
+  *
+  * This function will inform about known DN OIDs. This is useful since functions
+  * like gnutls_x509_crt_set_dn_by_oid() use the information on known
+  * OIDs to properly encode their input. Object Identifiers that are not
+  * known are not encoded by these functions, and their input is stored directly
+  * into the ASN.1 structure. In that case of unknown OIDs, you have
+  * the responsibility of DER encoding your data.
+  *
+  * Returns 1 on known OIDs and 0 otherwise.
+  *
+  **/
+int gnutls_x509_dn_oid_known( const char* oid) 
+{
+int i = 0;
+
+	do {
+		if ( strcmp(_oid2str[i].oid, oid)==0)
+			return 1;
+		i++;
+	} while( _oid2str[i].oid != NULL);
+
+	return 0;
+}
+
 /* Returns 1 if the data defined by the OID are of a choice
  * type.
  */
@@ -111,13 +138,11 @@ int i = 0;
  * hold the string.
  */
 int _gnutls_x509_oid_data2string( const char* oid, void* value, 
-	int value_size, char * res, size_t *res_size) {
-
-int result;
-char str[1024];
-char tmpname[128];
+	int value_size, char * res, size_t *res_size) 
+{
+char str[1024], tmpname[128];
 const char* ANAME = NULL;
-int CHOICE = -1, len = -1;
+int CHOICE = -1, len = -1, result;
 ASN1_TYPE tmpasn = ASN1_TYPE_EMPTY;
 
 	if (value==NULL || value_size <=0 || res_size == NULL) {
@@ -125,8 +150,6 @@ ASN1_TYPE tmpasn = ASN1_TYPE_EMPTY;
 		return GNUTLS_E_INVALID_REQUEST;
 	}
 	
-	if (res) res[0] = 0;
-
 	if ( _gnutls_x509_oid_data_printable( oid) == 0) {
 		gnutls_assert();
 		return GNUTLS_E_INTERNAL_ERROR;
@@ -151,6 +174,7 @@ ASN1_TYPE tmpasn = ASN1_TYPE_EMPTY;
 	}
 
 	if ((result = asn1_der_decoding(&tmpasn, value, value_size, NULL)) != ASN1_SUCCESS) {
+		gnutls_assert();
 		asn1_delete_structure(&tmpasn);
 		return _gnutls_asn2err(result);
 	}
@@ -160,6 +184,7 @@ ASN1_TYPE tmpasn = ASN1_TYPE_EMPTY;
 	 */
 	len = sizeof( str) - 1;
 	if ((result = asn1_read_value(tmpasn, "", str, &len)) != ASN1_SUCCESS) {	/* CHOICE */
+		gnutls_assert();
 		asn1_delete_structure(&tmpasn);
 		return _gnutls_asn2err(result);
 	}
@@ -602,7 +627,6 @@ int _gnutls_x509_export_int( ASN1_TYPE asn1_data,
 			*output_data_size = len;
 			if (result == ASN1_MEM_ERROR) {
 				gnutls_assert();
-				_gnutls_debug_log("Length required for der coding: %d\n", len);
 				return GNUTLS_E_SHORT_MEMORY_BUFFER;
 			}
 			gnutls_assert();
@@ -626,7 +650,6 @@ int _gnutls_x509_export_int( ASN1_TYPE asn1_data,
 		if ((result=asn1_der_coding( asn1_data, "", tmp, &len, NULL)) != ASN1_SUCCESS) {
 			gnutls_assert();
 			if (result == ASN1_MEM_ERROR) {
-				_gnutls_debug_log("Length required for der coding: %d\n", len);
 				*output_data_size = B64FSIZE(strlen(pem_header),len) + 1;
 			}
 			gnutls_afree(tmp);
