@@ -97,7 +97,10 @@ int gnutls_x509_crl_import(gnutls_x509_crl crl, const gnutls_datum * data,
 			   gnutls_x509_crt_fmt format)
 {
 	int result = 0, need_free = 0;
-	gnutls_datum _data = { data->data, data->size };
+	gnutls_datum _data;
+	
+	_data.data = data->data;
+	_data.size = data->size;
 
 	if (crl == NULL) {
 		gnutls_assert();
@@ -181,6 +184,7 @@ int gnutls_x509_crl_get_issuer_dn(gnutls_x509_crl crl, char *buf,
   * @crl: should contain a gnutls_x509_crl structure
   * @oid: holds an Object Identified in null terminated string
   * @indx: In case multiple same OIDs exist in the RDN, this specifies which to send. Use zero to get the first one.
+  * @raw_flag: If non zero returns the raw DER data of the DN part.
   * @buf: a pointer to a structure to hold the peer's name (may be null)
   * @sizeof_buf: initialy holds the size of 'buf'
   *
@@ -188,6 +192,9 @@ int gnutls_x509_crl_get_issuer_dn(gnutls_x509_crl crl, char *buf,
   * by the given OID. The output will be encoded as described in RFC2253.
   *
   * Some helper macros with popular OIDs can be found in gnutls/x509.h
+  * If raw flag is zero, this function will only return known OIDs as text. Other OIDs 
+  * will be DER encoded, as described in RFC2253 -- in hex format with a '\#' prefix.
+  * You can check about known OIDs using gnutls_x509_dn_oid_known().
   *
   * If buf is null then only the size will be filled.
   *
@@ -197,8 +204,8 @@ int gnutls_x509_crl_get_issuer_dn(gnutls_x509_crl crl, char *buf,
   *
   **/
 int gnutls_x509_crl_get_issuer_dn_by_oid(gnutls_x509_crl crl,
-					 const char *oid, int indx,
-					 char *buf, size_t *sizeof_buf)
+	const char *oid, int indx, unsigned int raw_flag,
+	void *buf, size_t *sizeof_buf)
 {
 	if (crl == NULL) {
 		gnutls_assert();
@@ -207,9 +214,39 @@ int gnutls_x509_crl_get_issuer_dn_by_oid(gnutls_x509_crl crl,
 
 	return _gnutls_x509_parse_dn_oid(crl->crl,
 					 "tbsCertList.issuer.rdnSequence",
-					 oid, indx, buf, sizeof_buf);
+					 oid, indx, raw_flag, buf, sizeof_buf);
 }
 
+/**
+  * gnutls_x509_crl_get_issuer_dn_oid - This function returns the Certificate request issuer's distinguished name OIDs
+  * @crl: should contain a gnutls_x509_crl structure
+  * @indx: Specifies which DN OID to send. Use zero to get the first one.
+  * @oid: a pointer to a structure to hold the name (may be null)
+  * @sizeof_oid: initialy holds the size of 'oid'
+  *
+  * This function will extract the requested OID of the name of the CRL issuer, specified
+  * by the given index. 
+  *
+  * If oid is null then only the size will be filled.
+  *
+  * Returns GNUTLS_E_SHORT_MEMORY_BUFFER if the provided buffer is not long enough, and
+  * in that case the sizeof_oid will be updated with the required size.
+  * On success 0 is returned.
+  *
+  **/
+int gnutls_x509_crl_get_dn_oid(gnutls_x509_crl crl, 
+	int indx, void *oid, size_t *sizeof_oid)
+{
+	if (crl==NULL) {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_REQUEST;
+	}
+
+	return _gnutls_x509_get_dn_oid( crl->crl, 
+		 "tbsCertList.issuer.rdnSequence", indx, oid, sizeof_oid);
+}
+
+
 /**
   * gnutls_x509_crl_get_signature_algorithm - This function returns the CRL's signature algorithm
   * @crl: should contain a gnutls_x509_crl structure
@@ -506,7 +543,7 @@ cleanup:
   *
   **/
 int gnutls_x509_crl_export( gnutls_x509_crl crl,
-	gnutls_x509_crt_fmt format, unsigned char* output_data, size_t* output_data_size)
+	gnutls_x509_crt_fmt format, void* output_data, size_t* output_data_size)
 {
 	if (crl == NULL) {
 		gnutls_assert();
