@@ -190,6 +190,7 @@ int gnutls_x509_crq_get_dn(gnutls_x509_crq crq, char *buf,
   * @crq: should contain a gnutls_x509_crq structure
   * @oid: holds an Object Identified in null terminated string
   * @indx: In case multiple same OIDs exist in the RDN, this specifies which to send. Use zero to get the first one.
+  * @raw_flag: If non zero returns the raw DER data of the DN part.
   * @buf: a pointer to a structure to hold the name (may be null)
   * @sizeof_buf: initialy holds the size of 'buf'
   *
@@ -197,6 +198,9 @@ int gnutls_x509_crq_get_dn(gnutls_x509_crq crq, char *buf,
   * by the given OID. The output will be encoded as described in RFC2253.
   *
   * Some helper macros with popular OIDs can be found in gnutls/x509.h
+  * If raw flag is zero, this function will only return known OIDs as text. Other OIDs
+  * will be DER encoded, as described in RFC2253 -- in hex format with a '\#' prefix.
+  * You can check about known OIDs using gnutls_x509_dn_oid_known().
   *
   * If buf is null then only the size will be filled.
   *
@@ -206,15 +210,44 @@ int gnutls_x509_crq_get_dn(gnutls_x509_crq crq, char *buf,
   *
   **/
 int gnutls_x509_crq_get_dn_by_oid(gnutls_x509_crq crq, const char* oid, 
-	int indx, char *buf, size_t *sizeof_buf)
+	int indx, unsigned int raw_flag, void *buf, size_t *sizeof_buf)
 {
 	if (crq==NULL) {
 		gnutls_assert();
 		return GNUTLS_E_INVALID_REQUEST;
 	}
 
-	return _gnutls_x509_parse_dn_oid( crq->crq, "certificationRequestInfo.subject.rdnSequence", oid,
-		indx, buf, sizeof_buf);
+	return _gnutls_x509_parse_dn_oid( crq->crq, "certificationRequestInfo.subject.rdnSequence", 
+		oid, indx, raw_flag, buf, sizeof_buf);
+}
+
+/**
+  * gnutls_x509_crq_get_dn_oid - This function returns the Certificate request subject's distinguished name OIDs
+  * @crq: should contain a gnutls_x509_crq structure
+  * @indx: Specifies which DN OID to send. Use zero to get the first one.
+  * @oid: a pointer to a structure to hold the name (may be null)
+  * @sizeof_oid: initialy holds the size of 'oid'
+  *
+  * This function will extract the requested OID of the name of the Certificate request subject, specified
+  * by the given index. 
+  *
+  * If oid is null then only the size will be filled.
+  *
+  * Returns GNUTLS_E_SHORT_MEMORY_BUFFER if the provided buffer is not long enough, and
+  * in that case the sizeof_oid will be updated with the required size.
+  * On success 0 is returned.
+  *
+  **/
+int gnutls_x509_crq_get_dn_oid(gnutls_x509_crq crq,
+	int indx, void *oid, size_t *sizeof_oid)
+{
+	if (crq==NULL) {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_REQUEST;
+	}
+
+	return _gnutls_x509_get_dn_oid( crq->crq, "certificationRequestInfo.subject.rdnSequence", 
+		indx, oid, sizeof_oid);
 }
 
 /* Parses an Attribute list in the asn1_struct, and searches for the
@@ -383,28 +416,32 @@ int gnutls_x509_crq_get_challenge_password(gnutls_x509_crq crq,
 /**
   * gnutls_x509_crq_set_dn_by_oid - This function will set the Certificate request subject's distinguished name
   * @crq: should contain a gnutls_x509_crq structure
-  * @oid: holds an Object Identified in null terminated string
-  * @name: a pointer to the name
-  * @sizeof_name: holds the size of 'name'
+  * @oid: holds an Object Identifier in a null terminated string
+  * @raw_flag: must be 0, or 1 if the data are DER encoded
+  * @data: a pointer to the input data
+  * @sizeof_data: holds the size of 'data'
   *
   * This function will set the part of the name of the Certificate request subject, specified
   * by the given OID. 
   *
   * Some helper macros with popular OIDs can be found in gnutls/x509.h
-  * With this function you can only set the known OIDs.
+  * With this function you can only set the known OIDs. You can test
+  * for known OIDs using gnutls_x509_dn_oid_known(). For OIDs that are
+  * not known (by gnutls) you should properly DER encode your data, and
+  * call this function with raw_flag set.
   *
   * Returns 0 on success.
   *
   **/
 int gnutls_x509_crq_set_dn_by_oid(gnutls_x509_crq crq, const char* oid, 
-	const char *name, unsigned int sizeof_name)
+	unsigned int raw_flag, const void *data, unsigned int sizeof_data)
 {
-	if (sizeof_name == 0 || name == NULL || crq == NULL) {
+	if (sizeof_data == 0 || data == NULL || crq == NULL) {
 		return GNUTLS_E_INVALID_REQUEST;
 	}
 	
-	return _gnutls_x509_set_dn_oid( crq->crq, "certificationRequestInfo.subject", oid,
-		name, sizeof_name);
+	return _gnutls_x509_set_dn_oid( crq->crq, "certificationRequestInfo.subject", 
+		oid, raw_flag, data, sizeof_data);
 }
 
 /**
@@ -606,7 +643,7 @@ const char* pk;
   *
   **/
 int gnutls_x509_crq_export( gnutls_x509_crq crq,
-	gnutls_x509_crt_fmt format, unsigned char* output_data, size_t* output_data_size)
+	gnutls_x509_crt_fmt format, void* output_data, size_t* output_data_size)
 {
 	if (crq==NULL) {
 		gnutls_assert();
