@@ -97,6 +97,11 @@ void gnutls_pkcs12_bag_deinit(gnutls_pkcs12_bag bag)
   **/
 gnutls_pkcs12_bag_type gnutls_pkcs12_bag_get_type(gnutls_pkcs12_bag bag, int indx)
 {
+	if (bag == NULL) {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_REQUEST;
+	}
+
 	if (indx >= bag->bag_elements) 
 		return GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE;
 	return bag->element[indx].type;
@@ -111,6 +116,11 @@ gnutls_pkcs12_bag_type gnutls_pkcs12_bag_get_type(gnutls_pkcs12_bag bag, int ind
   **/
 int gnutls_pkcs12_bag_get_count(gnutls_pkcs12_bag bag)
 {
+	if (bag == NULL) {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_REQUEST;
+	}
+
 	return bag->bag_elements;
 }
 
@@ -125,6 +135,11 @@ int gnutls_pkcs12_bag_get_count(gnutls_pkcs12_bag bag)
   **/
 int gnutls_pkcs12_bag_get_data(gnutls_pkcs12_bag bag, int indx, gnutls_const_datum * data)
 {
+	if (bag == NULL) {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_REQUEST;
+	}
+
 	if (indx >= bag->bag_elements) 
 		return GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE;
 
@@ -273,7 +288,9 @@ int _pkcs12_encode_crt_bag( gnutls_pkcs12_bag_type type, const gnutls_datum* raw
   * @data: the data to be copied.
   *
   * This function will insert the given data of the given type into the
-  * bag. On success returns the index of the added bag, or a negative
+  * bag. 
+  *
+  * Returns the index of the added bag on success, or a negative
   * value on error.
   *
   **/
@@ -281,6 +298,10 @@ int gnutls_pkcs12_bag_set_data(gnutls_pkcs12_bag bag, gnutls_pkcs12_bag_type typ
 	const gnutls_datum* data)
 {
 int ret;
+	if (bag == NULL) {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_REQUEST;
+	}
 
 	if (bag->bag_elements == MAX_BAG_ELEMENTS-1) {
 		gnutls_assert();
@@ -323,8 +344,8 @@ int ret;
   * This function will insert the given certificate into the
   * bag. This is just a wrapper over gnutls_pkcs12_bag_set_data().
   *
-  * On success returns the index of the added bag, or a negative
-  * value on error.
+  * Returns the index of the added bag on success, or a negative
+  * value on failure.
   *
   **/
 int gnutls_pkcs12_bag_set_crt(gnutls_pkcs12_bag bag, gnutls_x509_crt crt)
@@ -332,6 +353,11 @@ int gnutls_pkcs12_bag_set_crt(gnutls_pkcs12_bag bag, gnutls_x509_crt crt)
 int ret;
 gnutls_datum data;
 
+	if (bag == NULL) {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_REQUEST;
+	}
+
 	ret = _gnutls_x509_der_encode( crt->cert, "", &data, 0);
 	if (ret < 0) {
 		gnutls_assert();
@@ -353,8 +379,8 @@ gnutls_datum data;
   * This function will insert the given CRL into the
   * bag. This is just a wrapper over gnutls_pkcs12_bag_set_data().
   *
-  * On success returns the index of the added bag, or a negative
-  * value on error.
+  * Returns the index of the added bag on success, or a negative
+  * value on failure.
   *
   **/
 int gnutls_pkcs12_bag_set_crl(gnutls_pkcs12_bag bag, gnutls_x509_crl crl)
@@ -362,6 +388,12 @@ int gnutls_pkcs12_bag_set_crl(gnutls_pkcs12_bag bag, gnutls_x509_crl crl)
 int ret;
 gnutls_datum data;
 
+
+	if (bag == NULL) {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_REQUEST;
+	}
+
 	ret = _gnutls_x509_der_encode( crl->crl, "", &data, 0);
 	if (ret < 0) {
 		gnutls_assert();
@@ -393,6 +425,12 @@ int gnutls_pkcs12_bag_set_key_id(gnutls_pkcs12_bag bag, int indx,
 {
 int ret;
 
+
+	if (bag == NULL) {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_REQUEST;
+	}
+
 	if (indx > bag->bag_elements-1) {
 		gnutls_assert();
 		return GNUTLS_E_INVALID_REQUEST;
@@ -424,6 +462,10 @@ int ret;
 int gnutls_pkcs12_bag_get_key_id(gnutls_pkcs12_bag bag, int indx, 
 	gnutls_datum* id)
 {
+	if (bag == NULL) {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_REQUEST;
+	}
 
 	if (indx > bag->bag_elements-1) {
 		gnutls_assert();
@@ -451,6 +493,11 @@ int gnutls_pkcs12_bag_get_key_id(gnutls_pkcs12_bag bag, int indx,
 int gnutls_pkcs12_bag_get_friendly_name(gnutls_pkcs12_bag bag, int indx, 
 	char **name)
 {
+	if (bag == NULL) {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_REQUEST;
+	}
+
 	if (indx > bag->bag_elements-1) {
 		gnutls_assert();
 		return GNUTLS_E_INVALID_REQUEST;
@@ -478,6 +525,11 @@ int gnutls_pkcs12_bag_get_friendly_name(gnutls_pkcs12_bag bag, int indx,
 int gnutls_pkcs12_bag_set_friendly_name(gnutls_pkcs12_bag bag, int indx, 
 	const char* name)
 {
+	if (bag == NULL) {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_REQUEST;
+	}
+
 	if (indx > bag->bag_elements-1) {
 		gnutls_assert();
 		return GNUTLS_E_INVALID_REQUEST;
@@ -506,6 +558,11 @@ int gnutls_pkcs12_bag_decrypt(gnutls_pkcs12_bag bag, const char* pass)
 {
 int ret;
 gnutls_datum dec;
+
+	if (bag == NULL) {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_REQUEST;
+	}
 	
 	if (bag->element[0].type != GNUTLS_BAG_ENCRYPTED) {
 		gnutls_assert();
@@ -555,6 +612,11 @@ gnutls_datum der = {NULL, 0};
 gnutls_datum enc = {NULL, 0};
 schema_id id;
 
+	if (bag == NULL) {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_REQUEST;
+	}
+
 	if (bag->element[0].type == GNUTLS_BAG_ENCRYPTED) {
 		gnutls_assert();
 		return GNUTLS_E_INVALID_REQUEST;
