@@ -399,7 +399,7 @@ int gnutls_x509_privkey_export_pkcs8(gnutls_x509_privkey key,
 				     gnutls_x509_crt_fmt format,
 				     const char *password,
 				     unsigned int flags,
-				     unsigned char *output_data,
+				     void *output_data,
 				     size_t *output_data_size)
 {
 	ASN1_TYPE pkcs8_asn, pkey_info;
