@@ -242,6 +242,7 @@ int gnutls_x509_crt_get_issuer_dn(gnutls_x509_crt cert, char *buf,
   * @cert: should contain a gnutls_x509_crt structure
   * @oid: holds an Object Identified in null terminated string
   * @indx: In case multiple same OIDs exist in the RDN, this specifies which to send. Use zero to get the first one.
+  * @raw_flag: If non zero returns the raw DER data of the DN part.
   * @buf: a pointer to a structure to hold the name (may be null)
   * @sizeof_buf: initialy holds the size of 'buf'
   *
@@ -249,6 +250,9 @@ int gnutls_x509_crt_get_issuer_dn(gnutls_x509_crt cert, char *buf,
   * by the given OID. The output will be encoded as described in RFC2253.
   *
   * Some helper macros with popular OIDs can be found in gnutls/x509.h
+  * If raw flag is zero, this function will only return known OIDs as text. Other OIDs
+  * will be DER encoded, as described in RFC2253 -- in hex format with a '\#' prefix.
+  * You can check about known OIDs using gnutls_x509_dn_oid_known().
   *
   * If buf is null then only the size will be filled.
   *
@@ -258,7 +262,7 @@ int gnutls_x509_crt_get_issuer_dn(gnutls_x509_crt cert, char *buf,
   *
   **/
 int gnutls_x509_crt_get_issuer_dn_by_oid(gnutls_x509_crt cert, const char* oid, 
-	int indx, char *buf, size_t *sizeof_buf)
+	int indx, unsigned int raw_flag, void *buf, size_t *sizeof_buf)
 {
 	if (cert==NULL) {
 		gnutls_assert();
@@ -266,7 +270,36 @@ int gnutls_x509_crt_get_issuer_dn_by_oid(gnutls_x509_crt cert, const char* oid,
 	}
 
 	return _gnutls_x509_parse_dn_oid( cert->cert, "tbsCertificate.issuer.rdnSequence", oid,
-		indx, buf, sizeof_buf);
+		indx, raw_flag, buf, sizeof_buf);
+}
+
+/**
+  * gnutls_x509_crt_get_issuer_dn_oid - This function returns the Certificate's issuer distinguished name OIDs
+  * @cert: should contain a gnutls_x509_crt structure
+  * @indx: This specifies which OID to return. Use zero to get the first one.
+  * @oid: a pointer to a buffer to hold the OID (may be null)
+  * @sizeof_oid: initialy holds the size of 'oid'
+  *
+  * This function will extract the OIDs of the name of the Certificate issuer specified
+  * by the given index.
+  *
+  * If oid is null then only the size will be filled.
+  *
+  * Returns GNUTLS_E_SHORT_MEMORY_BUFFER if the provided buffer is not long enough, and
+  * in that case the sizeof_oid will be updated with the required size.
+  * On success 0 is returned.
+  *
+  **/
+int gnutls_x509_crt_get_issuer_dn_oid(gnutls_x509_crt cert, 
+	int indx, void *oid, size_t *sizeof_oid)
+{
+	if (cert==NULL) {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_REQUEST;
+	}
+
+	return _gnutls_x509_get_dn_oid( cert->cert, "tbsCertificate.issuer.rdnSequence", 
+		indx, oid, sizeof_oid);
 }
 
 /**
@@ -302,13 +335,17 @@ int gnutls_x509_crt_get_dn(gnutls_x509_crt cert, char *buf,
   * @cert: should contain a gnutls_x509_crt structure
   * @oid: holds an Object Identified in null terminated string
   * @indx: In case multiple same OIDs exist in the RDN, this specifies which to send. Use zero to get the first one.
+  * @raw_flag: If non zero returns the raw DER data of the DN part.
   * @buf: a pointer to a structure to hold the name (may be null)
   * @sizeof_buf: initialy holds the size of 'buf'
   *
   * This function will extract the part of the name of the Certificate subject, specified
-  * by the given OID. The output will be encoded as described in RFC2253.
+  * by the given OID.
   *
   * Some helper macros with popular OIDs can be found in gnutls/x509.h
+  * If raw flag is zero, this function will only return known OIDs as text. Other OIDs
+  * will be DER encoded, as described in RFC2253 -- in hex format with a '\#' prefix.
+  * You can check about known OIDs using gnutls_x509_dn_oid_known().
   *
   * If buf is null then only the size will be filled.
   *
@@ -318,7 +355,7 @@ int gnutls_x509_crt_get_dn(gnutls_x509_crt cert, char *buf,
   *
   **/
 int gnutls_x509_crt_get_dn_by_oid(gnutls_x509_crt cert, const char* oid, 
-	int indx, char *buf, size_t *sizeof_buf)
+	int indx, unsigned int raw_flag, void *buf, size_t *sizeof_buf)
 {
 	if (cert==NULL) {
 		gnutls_assert();
@@ -326,7 +363,36 @@ int gnutls_x509_crt_get_dn_by_oid(gnutls_x509_crt cert, const char* oid,
 	}
 
 	return _gnutls_x509_parse_dn_oid( cert->cert, "tbsCertificate.subject.rdnSequence", oid,
-		indx, buf, sizeof_buf);
+		indx, raw_flag, buf, sizeof_buf);
+}
+
+/**
+  * gnutls_x509_crt_get_dn_oid - This function returns the Certificate's subject distinguished name OIDs
+  * @cert: should contain a gnutls_x509_crt structure
+  * @indx: This specifies which OID to return. Use zero to get the first one.
+  * @oid: a pointer to a buffer to hold the OID (may be null)
+  * @sizeof_oid: initialy holds the size of 'oid'
+  *
+  * This function will extract the OIDs of the name of the Certificate subject specified
+  * by the given index.
+  *
+  * If oid is null then only the size will be filled.
+  *
+  * Returns GNUTLS_E_SHORT_MEMORY_BUFFER if the provided buffer is not long enough, and
+  * in that case the sizeof_oid will be updated with the required size.
+  * On success 0 is returned.
+  *
+  **/
+int gnutls_x509_crt_get_dn_oid(gnutls_x509_crt cert, 
+	int indx, void *oid, size_t *sizeof_oid)
+{
+	if (cert==NULL) {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_REQUEST;
+	}
+
+	return _gnutls_x509_get_dn_oid( cert->cert, "tbsCertificate.subject.rdnSequence", 
+		indx, oid, sizeof_oid);
 }
 
 /**
@@ -450,7 +516,7 @@ time_t gnutls_x509_crt_get_expiration_time(gnutls_x509_crt cert)
   * Returns 0 on success and a negative value in case of an error.
   *
   **/
-int gnutls_x509_crt_get_serial(gnutls_x509_crt cert, char* result, 
+int gnutls_x509_crt_get_serial(gnutls_x509_crt cert, void* result, 
 	size_t* result_size)
 {
 	int ret;
@@ -519,7 +585,7 @@ int gnutls_x509_crt_get_pk_algorithm( gnutls_x509_crt cert, unsigned int* bits)
   * given certificate.
   * 
   * This is specified in X509v3 Certificate Extensions. 
-  * GNUTLS will return the Alternative name, or a negative
+  * GNUTLS will return the Alternative name (2.5.29.17), or a negative
   * error code.
   *
   * Returns GNUTLS_E_SHORT_MEMORY_BUFFER if ret_size is not enough to hold the alternative 
@@ -531,7 +597,7 @@ int gnutls_x509_crt_get_pk_algorithm( gnutls_x509_crt cert, unsigned int* bits)
   *
   **/
 int gnutls_x509_crt_get_subject_alt_name(gnutls_x509_crt cert, 
-	int seq, char *ret, size_t *ret_size, unsigned int *critical)
+	int seq, void *ret, size_t *ret_size, unsigned int *critical)
 {
 	int result;
 	gnutls_datum dnsname;
@@ -635,7 +701,7 @@ int gnutls_x509_crt_get_subject_alt_name(gnutls_x509_crt cert,
   * @critical: will be non zero if the extension is marked as critical
   *
   * This function will return certificates CA status, by reading the
-  * basicConstraints X.509 extension. If the certificate is a CA a positive
+  * basicConstraints X.509 extension (2.5.29.19). If the certificate is a CA a positive
   * value will be returned, or zero if the certificate does not have
   * CA flag set. 
   *
@@ -685,7 +751,7 @@ int gnutls_x509_crt_get_ca_status(gnutls_x509_crt cert, unsigned int* critical)
   * @critical: will be non zero if the extension is marked as critical
   *
   * This function will return certificate's key usage, by reading the 
-  * keyUsage X.509 extension. The key usage value will ORed values of the:
+  * keyUsage X.509 extension (2.5.29.15). The key usage value will ORed values of the:
   * GNUTLS_KEY_DIGITAL_SIGNATURE, GNUTLS_KEY_NON_REPUDIATION,
   * GNUTLS_KEY_KEY_ENCIPHERMENT, GNUTLS_KEY_DATA_ENCIPHERMENT,
   * GNUTLS_KEY_KEY_AGREEMENT, GNUTLS_KEY_KEY_CERT_SIGN,
@@ -752,7 +818,7 @@ int gnutls_x509_crt_get_key_usage(gnutls_x509_crt cert, unsigned int *key_usage,
   *
   **/
 int gnutls_x509_crt_get_extension_by_oid(gnutls_x509_crt cert, const char* oid,
-	int indx, unsigned char* buf, size_t * sizeof_buf, unsigned int * critical)
+	int indx, void* buf, size_t * sizeof_buf, unsigned int * critical)
 {
 	int result;
 	gnutls_datum output;
@@ -790,6 +856,41 @@ int gnutls_x509_crt_get_extension_by_oid(gnutls_x509_crt cert, const char* oid,
 	
 }
 
+/**
+  * gnutls_x509_crt_get_extension_oid - This function returns the specified extension OID
+  * @cert: should contain a gnutls_x509_crt structure
+  * @indx: Specifies which extension OID to send. Use zero to get the first one.
+  * @oid: a pointer to a structure to hold the OID (may be null)
+  * @sizeof_oid: initialy holds the size of 'oid'
+  *
+  * This function will return the requested extension OID in the certificate.
+  * The extension OID will be stored as a string in the provided buffer.
+  *
+  * A negative value may be returned in case of parsing error.
+  * If your have reached the last extension available 
+  * GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE will be returned.
+  *
+  **/
+int gnutls_x509_crt_get_extension_oid(gnutls_x509_crt cert, int indx, 
+	void* oid, size_t * sizeof_oid)
+{
+	int result;
+
+	if (cert==NULL) {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_REQUEST;
+	}
+
+	if ((result =
+	     _gnutls_x509_crt_get_extension_oid(cert, indx, oid, sizeof_oid)) < 0) {
+	     	gnutls_assert();
+		return result;
+	}
+
+	return 0;
+	
+}
+
 
 static
 int _gnutls_x509_crt_get_raw_dn2( gnutls_x509_crt cert,
@@ -899,7 +1000,7 @@ int _gnutls_x509_crt_get_raw_dn( gnutls_x509_crt cert,
   *
   **/
 int gnutls_x509_crt_get_fingerprint(gnutls_x509_crt cert, 
-	gnutls_digest_algorithm algo, unsigned char *buf,
+	gnutls_digest_algorithm algo, void *buf,
 	 size_t *sizeof_buf)
 {
 opaque *cert_buf;
@@ -958,7 +1059,7 @@ gnutls_datum tmp;
   *
   **/
 int gnutls_x509_crt_export( gnutls_x509_crt cert,
-	gnutls_x509_crt_fmt format, unsigned char* output_data, size_t* output_data_size)
+	gnutls_x509_crt_fmt format, void* output_data, size_t* output_data_size)
 {
 	if (cert==NULL) {
 		gnutls_assert();
