@@ -44,6 +44,7 @@ gnutls_x509_crt load_cert(int mand);
 void certificate_info( void);
 void crl_info( void);
 void privkey_info( void);
+static void print_certificate_info( gnutls_x509_crt crt);
 static void gaa_parser(int argc, char **argv);
 void generate_self_signed( void);
 void generate_request(void);
@@ -81,7 +82,7 @@ int ret;
 	
 	if (strlen(input)==1) /* only newline */ return;
 
-	ret = gnutls_x509_crt_set_dn_by_oid(crt, oid, input, strlen(input)-1);
+	ret = gnutls_x509_crt_set_dn_by_oid(crt, oid, 0, input, strlen(input)-1);
 	if (ret < 0) {
 		fprintf(stderr, "set_dn: %s\n", gnutls_strerror(ret));
 		exit(1);
@@ -98,7 +99,7 @@ int ret;
 	
 	if (strlen(input)==1) /* only newline */ return;
 
-	ret = gnutls_x509_crq_set_dn_by_oid(crq, oid, input, strlen(input)-1);
+	ret = gnutls_x509_crq_set_dn_by_oid(crq, oid, 0, input, strlen(input)-1);
 	if (ret < 0) {
 		fprintf(stderr, "set_dn: %s\n", gnutls_strerror(ret));
 		exit(1);
@@ -117,10 +118,6 @@ char input[128];
 	return atoi(input);
 }
 
-#ifdef _WIN32
-# define getpass read_str
-#endif
-
 static const char* read_str( const char* input_str)
 {
 static char input[128];
@@ -135,6 +132,24 @@ static char input[128];
 	return input;
 }
 
+static const char* read_pass( const char* input_str)
+{
+#ifdef _WIN32
+static char input[128];
+
+	fputs( input_str, stderr);
+	fgets( input, sizeof(input), stdin);
+	
+	input[strlen(input)-1] = 0;
+
+	if (strlen(input)==0) return NULL;
+
+	return input;
+#else
+	return getpass(input_str);
+#endif
+}
+
 static int read_yesno( const char* input_str)
 {
 char input[128];
@@ -198,7 +213,9 @@ static void print_key_usage( unsigned int x)
 
 static void print_private_key( gnutls_x509_privkey key)
 {
-int size, ret;
+int ret;
+size_t size;
+
 	if (!key) return;
 
 	if (!info.pkcs8) {
@@ -295,7 +312,7 @@ gnutls_x509_crt generate_certificate( gnutls_x509_privkey *ret_key)
 		exit(1);
 	}
 
-	serial = read_int( "Enter the certificate's serial number: ");
+	serial = read_int( "Enter the certificate's serial number (decimal): ");
 	buffer[2] = serial & 0xff;
 	buffer[1] = (serial >> 8) & 0xff;
 	buffer[0] = (serial >> 16) & 0xff;
@@ -395,13 +412,15 @@ void generate_self_signed( void)
 {
 	gnutls_x509_crt crt;
 	gnutls_x509_privkey key;
-	int size;
+	size_t size;
 	int result;
 
 	fprintf(stderr, "Generating a self signed certificate...\n");
 
 	crt = generate_certificate( &key);
 
+	print_certificate_info( crt);
+
 	fprintf(stderr, "\n\nSigning certificate...\n");
 
 	result = gnutls_x509_crt_sign( crt, crt, key);
@@ -427,7 +446,8 @@ void generate_signed_certificate( void)
 {
 	gnutls_x509_crt crt;
 	gnutls_x509_privkey key;
-	int size, result;
+	size_t size;
+	int result;
 	gnutls_x509_privkey ca_key;
 	gnutls_x509_crt ca_crt;
 
@@ -437,6 +457,8 @@ void generate_signed_certificate( void)
 	ca_crt = load_ca_cert();
 
 	crt = generate_certificate( &key);
+
+	print_certificate_info( crt);
 	
 	fprintf(stderr, "\n\nSigning certificate...\n");
 
@@ -462,7 +484,8 @@ void generate_signed_certificate( void)
 void update_signed_certificate( void)
 {
 	gnutls_x509_crt crt;
-	int size, result;
+	size_t size;
+	int result;
 	gnutls_x509_privkey ca_key;
 	gnutls_x509_crt ca_crt;
 
@@ -592,19 +615,31 @@ const char* get_algorithm( int a)
 	}
 }
 
+static inline int known_oid( const char* oid)
+{
+	if (strcmp(oid, "2.5.29.17") == 0 ||
+		strcmp( oid, "2.5.29.19") == 0 ||
+		strcmp( oid, "2.5.29.15") == 0)
+			return 1;
+	
+	return 0;
+}
+
 void certificate_info( void)
 {
 	gnutls_x509_crt crt;
-	int size, ret, i;
+	int ret, i, indx, j;
 	unsigned int critical, key_usage;
 	time_t tim;
 	gnutls_datum pem;
 	char serial[40];
-	size_t serial_size = sizeof(serial), dn_size;
+	size_t serial_size = sizeof(serial), dn_size, size;
 	char printable[256];
 	char *print;
 	const char* cprint;
 	char dn[256];
+	char oid[128] = "";
+	char old_oid[128] = "";
 		
 	size = fread( buffer, 1, sizeof(buffer)-1, infile);
 	buffer[size] = 0;
@@ -619,6 +654,8 @@ void certificate_info( void)
 		fprintf(stderr, "Decoding error: %s\n", gnutls_strerror(ret));
 		exit(1);
 	}
+
+	fprintf(outfile, "X.509 certificate info:\n\n");
 	
 	fprintf(outfile, "Version: %d\n", gnutls_x509_crt_get_version(crt));
 
@@ -631,7 +668,7 @@ void certificate_info( void)
 				(unsigned char) serial[i]);
 			print += 3;
 		}
-		fprintf(outfile, "Serial Number: %s\n", printable);
+		fprintf(outfile, "Serial Number (hex): %s\n", printable);
 	}
 	
 
@@ -682,6 +719,7 @@ void certificate_info( void)
 	
 	/* subject alternative name
 	 */
+	ret = 0;
 	for (i = 0; !(ret < 0); i++) {
 		size = sizeof(buffer);
 		ret = gnutls_x509_crt_get_subject_alt_name(crt, i, buffer, &size, &critical);
@@ -733,6 +771,48 @@ void certificate_info( void)
 		print_key_usage(key_usage);
 	}
 
+	/* other extensions:
+	 */
+	indx = 0;
+	ret = 0;
+	for (i = 0; !(ret < 0); i++) {
+
+		size = sizeof(oid);
+		ret = gnutls_x509_crt_get_extension_oid( crt, i, oid, &size);
+
+		if (ret >= 0) {
+			if (known_oid( oid)) continue;
+
+			if (strcmp( oid, old_oid) == 0) {
+				indx++;
+			} else {
+				indx = 0;
+			}
+
+			fprintf( outfile, "\n\t%s: ", oid);
+			
+			size = sizeof(buffer);
+			ret = gnutls_x509_crt_get_extension_by_oid( crt, oid, indx, buffer, &size, &critical);
+			if (ret >= 0) {
+				if (critical)
+					fprintf(outfile, "(critical)\n");
+				else
+					fprintf(outfile, "\n");
+
+				print = printable;
+				for (j = 0; j < size; j++) {
+					sprintf(print, "%.2x ", (unsigned char) buffer[j]);
+					print += 3;
+				}
+				fprintf(outfile, "\t\tDER Data: %s\n", printable);
+			
+			}
+	
+			ret = 0;
+			strcpy( old_oid, oid);
+		}
+	}
+
 
 	/* fingerprint
 	 */
@@ -769,13 +849,132 @@ void certificate_info( void)
 	fprintf(outfile, "\n");
 }
 
+static void print_certificate_info( gnutls_x509_crt crt)
+{
+	int ret, i;
+	unsigned int critical, key_usage;
+	time_t tim;
+	char serial[40];
+	size_t serial_size = sizeof(serial), dn_size, size;
+	char printable[256];
+	char *print;
+	const char* cprint;
+	char dn[256];
+	
+	fprintf( stderr, "\n\nX.509 certificate info:\n\n");
+	
+	fprintf(stderr, "Version: %d\n", gnutls_x509_crt_get_version(crt));
+
+	/* serial number
+	 */
+	if (gnutls_x509_crt_get_serial(crt, serial, &serial_size) >= 0) {
+		print = printable;
+		for (i = 0; i < serial_size; i++) {
+			sprintf(print, "%.2x ",
+				(unsigned char) serial[i]);
+			print += 3;
+		}
+		fprintf(stderr, "Serial Number (hex): %s\n", printable);
+	}
+	
+	/* Validity
+	 */
+	fprintf(stderr, "Validity:\n");
+
+	tim = gnutls_x509_crt_get_activation_time(crt);
+	fprintf(stderr, "\tNot Before: %s", ctime(&tim));
+
+	tim = gnutls_x509_crt_get_expiration_time(crt);
+	fprintf(stderr, "\tNot After: %s", ctime(&tim));
+
+	/* Subject
+	 */
+	dn_size = sizeof(dn);
+	ret = gnutls_x509_crt_get_dn(crt, dn, &dn_size);
+	if (ret >= 0)
+		fprintf(stderr, "Subject: %s\n", dn);
+
+	/* Public key algorithm
+	 */
+	fprintf(stderr, "Subject Public Key Info:\n");
+	ret = gnutls_x509_crt_get_pk_algorithm(crt, NULL);
+	fprintf(stderr, "\tPublic Key Algorithm: ");
+
+	cprint = get_algorithm( ret);
+	fprintf(stderr,  "%s\n", cprint);
+
+
+	
+	fprintf(stderr, "\nX.509 Extensions:\n");
+	
+	/* subject alternative name
+	 */
+	for (i = 0; !(ret < 0); i++) {
+		size = sizeof(buffer);
+		ret = gnutls_x509_crt_get_subject_alt_name(crt, i, buffer, &size, &critical);
+
+		if (i==0 && ret != GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE) {
+			fprintf(stderr, "\tSubject Alternative name:");
+			if (critical) fprintf(stderr, " (critical)");
+			fprintf(stderr, "\n");
+		}
+		
+		if (ret < 0 && ret != GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE) {
+			fprintf(stderr, "\t\tFound unsupported alternative name.\n");
+		} else switch (ret) {
+			case GNUTLS_SAN_DNSNAME:
+				fprintf(stderr, "\t\tDNSname: %s\n", buffer);
+				break;
+			case GNUTLS_SAN_RFC822NAME:
+				fprintf(stderr, "\t\tRFC822name: %s\n", buffer);
+				break;
+			case GNUTLS_SAN_URI:
+				fprintf(stderr, "\t\tURI: %s\n", buffer);
+				break;
+			case GNUTLS_SAN_IPADDRESS:
+				fprintf(stderr, "\t\tIPAddress: %s\n", buffer);
+				break;
+		}
+	}
+	
+	/* check for basicConstraints
+	 */
+	ret = gnutls_x509_crt_get_ca_status( crt, &critical);
+	
+	if (ret >= 0) {
+		fprintf(stderr, "\tBasic Constraints:");
+		if (critical) fprintf(stderr, " (critical)");
+		fprintf(stderr, "\n");		
+
+		if (ret==0) fprintf(stderr, "\t\tCA:FALSE\n");
+		else fprintf(stderr, "\t\tCA:TRUE\n");
+		
+	}
+
+	/* Key Usage.
+	 */
+	ret = gnutls_x509_crt_get_key_usage( crt, &key_usage, &critical);
+	
+	if (ret >= 0) {
+		fprintf(stderr, "\tKey usage:\n");
+		print_key_usage(key_usage);
+	}
+
+	fprintf(stderr, "\n");
+
+	if (read_yesno( "Is the above information ok? (Y/N): ")==0) {
+		exit(1);
+	}
+}
+
 void crl_info(void)
 {
 	gnutls_x509_crl crl;
-	int size, ret, i, rc;
+	int ret, i, rc;
+	size_t size;
 	time_t tim;
 	gnutls_datum pem;
-	char serial[40];
+	unsigned char serial[40];
 	size_t serial_size = sizeof(serial), dn_size;
 	char printable[256];
 	char *print;
@@ -851,7 +1050,8 @@ void crl_info(void)
 void privkey_info( void)
 {
 	gnutls_x509_privkey key;
-	int size, ret, i;
+	size_t size;
+	int ret, i;
 	gnutls_datum pem;
 	char printable[256];
 	char *print;
@@ -1175,7 +1375,7 @@ void generate_request(void)
 		exit(1);
 	}
 
-	pass = read_str("Enter a challenge password: ");
+	pass = read_pass("Enter a challenge password: ");
 	
 	if (pass != NULL) {
 		ret = gnutls_x509_crq_set_challenge_password( crq, pass);
@@ -1216,7 +1416,7 @@ static void print_verification_res( gnutls_x509_crt crt, gnutls_x509_crt issuer,
 
 #define CERT_SEP "-----BEGIN CERT"
 #define CRL_SEP "-----BEGIN X509 CRL"
-int _verify_x509_mem( const char* cert, int cert_size)
+int _verify_x509_mem( const void* cert, int cert_size)
 {
 	int siz, i;
 	const char *ptr;
@@ -1506,7 +1706,7 @@ void generate_pkcs12( void)
 	int result;
 	size_t size;
 	gnutls_datum data;
-	char* password;
+	const char* password;
 	const char* name;
 	unsigned int flags;
 	gnutls_datum key_id;
@@ -1523,7 +1723,7 @@ void generate_pkcs12( void)
 	} while( name == NULL);
 
 	if (info.pass) password = info.pass;
-	else password = getpass( "Enter password: ");
+	else password = read_pass( "Enter password: ");
 
 	result = gnutls_pkcs12_bag_init( &bag);
 	if (result < 0) {
@@ -1747,7 +1947,7 @@ void pkcs12_info( void)
 	int result, ret;
 	size_t size;
 	gnutls_datum data;
-	char* password;
+	const char* password;
 	int index;
 	
 	size = fread( buffer, 1, sizeof(buffer)-1, infile);
@@ -1757,7 +1957,7 @@ void pkcs12_info( void)
 	data.size = size;
 
 	if (info.pass) password = info.pass;
-	else password = getpass( "Enter password: ");
+	else password = read_pass( "Enter password: ");
 	
 	result = gnutls_pkcs12_init(&pkcs12);
 	if (result < 0) {
