@@ -48,7 +48,7 @@ void print_x509_info(gnutls_session session, const char* hostname)
 	size_t serial_size = sizeof(serial);
 	char printable[256];
 	char *print;
-	int bits, algo;
+	unsigned int bits, algo;
 	time_t expiret, activet;
 
 	cert_list = gnutls_certificate_get_peers(session, &cert_list_size);
@@ -203,7 +203,8 @@ void print_openpgp_info(gnutls_session session, const char* hostname)
 {
 
 	char digest[20];
-	int digest_size = sizeof(digest), i, ret;
+	size_t digest_size = sizeof(digest);
+	int i, ret;
 	char printable[120];
 	char *print;
 	char name[256];
@@ -217,7 +218,7 @@ void print_openpgp_info(gnutls_session session, const char* hostname)
 	cert_list = gnutls_certificate_get_peers(session, &cert_list_size);
 
 	if (cert_list_size > 0) {
-		int algo, bits;
+		unsigned int algo, bits;
 
 		gnutls_openpgp_key_init(&crt);
 		ret =
@@ -486,6 +487,7 @@ void print_list(void)
 
 	printf("MACs:");
 	printf(" MD5");
+	printf(", RMD160");
 	printf(", SHA-1\n");
 
 	printf("Key exchange algorithms:");
@@ -571,6 +573,8 @@ void parse_macs(char **macs, int nmacs, int *mac_priority)
 		for (j = i = 0; i < nmacs; i++) {
 			if (strncasecmp(macs[i], "MD5", 3) == 0)
 				mac_priority[j++] = GNUTLS_MAC_MD5;
+			if (strncasecmp(macs[i], "RMD", 3) == 0)
+				mac_priority[j++] = GNUTLS_MAC_RMD160;
 			if (strncasecmp(macs[i], "SHA", 3) == 0)
 				mac_priority[j++] = GNUTLS_MAC_SHA;
 		}
